@@ -112,7 +112,7 @@ main (int argc, char *argv[])
   char *filename_out = NULL;
   Dwg_Version_Type dwg_version = R_2000;
   Bit_Chain dat = { 0 };
-  int do_free;
+  int do_free = 0;
   int need_free = 0;
   int c;
 #ifdef HAVE_GETOPT_LONG
@@ -122,6 +122,7 @@ main (int argc, char *argv[])
           { "file", 1, 0, 'o' },      { "as", 1, 0, 'a' },
           { "minimal", 0, 0, 'm' },   { "binary", 0, 0, 'b' },
           { "overwrite", 0, 0, 'y' }, { "help", 0, 0, 0 },
+          { "force-free", 0, 0, 0 },
           { "version", 0, 0, 0 },     { NULL, 0, NULL, 0 } };
 #endif
 
@@ -171,6 +172,8 @@ main (int argc, char *argv[])
             return opt_version ();
           if (!strcmp (long_options[option_index].name, "help"))
             return help ();
+          if (!strcmp (long_options[option_index].name, "force-free"))
+            do_free = 1;
           break;
 #else
         case 'i':
@@ -231,7 +234,7 @@ main (int argc, char *argv[])
       fprintf (stderr, "%s: no -o with multiple input files\n", argv[0]);
       return usage ();
     }
-  do_free = i + 1 < argc;
+  do_free |= (i + 1) < argc;
 
   while (i < argc)
     {
@@ -276,7 +279,7 @@ main (int argc, char *argv[])
       dat.from_version = dwg.header.from_version;
 
       if (minimal)
-        dwg.opts |= 0x10;
+        dwg.opts |= DWG_OPTS_MINIMAL;
       {
         struct stat attrib;
         if (!stat (filename_out, &attrib)) // exists
@@ -340,11 +343,18 @@ main (int argc, char *argv[])
         fclose (dat.fh);
 
     final:
+#if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
+  {
+    char *asanenv = getenv("ASAN_OPTIONS");
+    if (!asanenv)
+      do_free = 1;
+    // detect_leaks is enabled by default. see if it's turned off
+    else if (strstr (asanenv, "detect_leaks=0") == NULL) /* not found */
+      do_free = 1;
+  }
+#endif
       // forget about leaks. really huge DWG's need endlessly here.
       if (do_free
-#if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
-          || 1
-#endif
 #ifdef HAVE_VALGRIND_VALGRIND_H
           || (RUNNING_ON_VALGRIND)
 #endif
