@@ -284,15 +284,6 @@ main (int argc, char *argv[])
     }
   i = optind;
 
-#ifdef __AFL_HAVE_MANUAL_CONTROL
-  // llvm_mode deferred init
-  __AFL_INIT();
-#endif
-
-#ifdef __xxAFL_HAVE_MANUAL_CONTROL
-  while (__AFL_LOOP(1000)) { // llvm_mode persistent mode (currently broken)
-#endif
-  
   // get input format from INFILE, not outfile.
   // With stdin, should -I be mandatory, or try to autodetect the format?
   // With a file use the extension.
@@ -493,10 +484,6 @@ main (int argc, char *argv[])
       dwg_free (&dwg);
     }
 
-#ifdef __xxAFL_HAVE_MANUAL_CONTROL
-  } // __AFL_LOOP(1000) persistent mode
-#endif
-
   if (error >= DWG_ERR_CRITICAL)
     {
       fprintf (stderr, "ERROR 0x%x\n", error);
