@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009,2018-2020 Free Software Foundation, Inc.              */
+/*  Copyright (C) 2009,2018-2021 Free Software Foundation, Inc.              */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -27,6 +27,7 @@
 #include <stdint.h>
 #include <inttypes.h>
 #include <math.h>
+#include <assert.h>
 #if defined(HAVE_WCHAR_H) && defined(SIZEOF_WCHAR_T) && SIZEOF_WCHAR_T == 2
 #  include <wchar.h>
 #endif
@@ -491,12 +492,10 @@ bit_read_RD (Bit_Chain *dat)
   unsigned char byte[8];
   double *result;
 
+  //CHK_OVERFLOW_PLUS (8, __FUNCTION__, bit_nan ())
   // TODO: big-endian
   for (i = 0; i < 8; i++)
-    {
-      byte[i] = bit_read_RC (dat);
-      CHK_OVERFLOW(__FUNCTION__, bit_nan ())
-    }
+    byte[i] = bit_read_RC (dat);
 
   result = (double *)byte;
   return (*result);
@@ -1118,6 +1117,12 @@ bit_read_DD (Bit_Chain *dat, double default_value)
     }
 }
 
+int
+bit_eq_DD (double value, double default_value)
+{
+  return fabs (value - default_value) < 1e-12;
+}
+
 /** Write bit-double with default.
  */
 BITCODE_BB
@@ -1265,7 +1270,10 @@ void
 bit_write_H (Bit_Chain *restrict dat, Dwg_Handle *restrict handle)
 {
   int i;
-  unsigned char *val;
+  union {
+    unsigned char p[8];
+    unsigned long v;
+  } val;
   unsigned char size;
 
   if (!handle)
@@ -1279,11 +1287,12 @@ bit_write_H (Bit_Chain *restrict dat, Dwg_Handle *restrict handle)
       return;
     }
 
+  assert(sizeof(val) <= 8);
   // TODO: little-endian only. support sizes <= 8, not just 4
   memset (&val, 0, sizeof(val));
-  val = (unsigned char *)&handle->value;
+  val.v = handle->value;
   for (i = sizeof(val) - 1; i >= 0; i--)
-    if (val[i])
+    if (val.p[i])
       break;
 
   size = handle->code << 4;
@@ -1291,7 +1300,7 @@ bit_write_H (Bit_Chain *restrict dat, Dwg_Handle *restrict handle)
   bit_write_RC (dat, size);
 
   for (; i >= 0; i--)
-    bit_write_RC (dat, val[i]);
+    bit_write_RC (dat, val.p[i]);
 }
 
 /** Only read old 16bit CRC-numbers, without checking, only in order
@@ -1422,6 +1431,7 @@ bit_read_fixed (Bit_Chain *restrict dat, BITCODE_RC *restrict dest,
       loglevel = dat->opts & DWG_OPTS_LOGLEVEL;
       LOG_ERROR ("%s buffer overflow at pos %lu, size %lu", __FUNCTION__,
                  dat->byte, dat->size)
+      memset (dest, 0, length);
       return;
     }
   if (dat->bit == 0)
@@ -1504,6 +1514,17 @@ bit_read_bits (Bit_Chain *dat, unsigned long bits)
 void
 bit_write_TF (Bit_Chain *restrict dat, BITCODE_TF restrict chain, unsigned int length)
 {
+  if (!chain)
+    {
+      loglevel = dat->opts & DWG_OPTS_LOGLEVEL;
+      LOG_ERROR ("Empty TF with length %u", length);
+      if (length <= 128) // either chain or length is wrong
+        {
+          for (unsigned int i = 0; i < length; i++)
+            bit_write_RC (dat, 0);
+        }
+      return;
+    }
   if (dat->bit == 0 && dat->byte + length < dat->size)
     {
       memcpy (&dat->chain[dat->byte], chain, length);
@@ -1518,6 +1539,7 @@ bit_write_TF (Bit_Chain *restrict dat, BITCODE_TF restrict chain, unsigned int l
 
 /** Read simple text. After usage, the allocated memory must be properly freed.
  */
+ATTRIBUTE_MALLOC
 BITCODE_TV
 bit_read_TV (Bit_Chain *restrict dat)
 {
@@ -1537,9 +1559,14 @@ bit_read_TV (Bit_Chain *restrict dat)
     }
   for (i = 0; i < length; i++)
     chain[i] = bit_read_RC (dat);
-  // normally not needed, as the DWG itself contains the ending \0 as last char
+  // check if the string is already zero-terminated or not.
+  // only observed >=r2004 as writer app
+  if (length > 0 && dat->from_version > R_2000 && chain[length - 1] != '\0')
+    LOG_HANDLE ("TV-not-ZERO %u\n ", length)
+  else if (length > 0 && dat->from_version <= R_2000 && chain[length - 1] == '\0')
+    LOG_HANDLE ("TV-ZERO %u\n", length)
+  // normally not needed, as the DWG since r2004 itself contains the ending \0 as last char
   chain[i] = '\0';
-
   return (char *)chain;
 }
 
@@ -1554,6 +1581,7 @@ static int heX (unsigned char c)
    Max len widechars.
    No codepage support yet, only cp 30.
  */
+ATTRIBUTE_MALLOC
 char *
 bit_embed_TU_size (BITCODE_TU restrict wstr, const int len)
 {
@@ -1798,12 +1826,16 @@ bit_embed_TU (BITCODE_TU restrict wstr)
 }
 
 /** Write ASCIIZ text.
+    Starting with r2004 as writer (not target version) acad always
+    writes a terminating zero.
  */
 void
 bit_write_TV (Bit_Chain *restrict dat, BITCODE_TV restrict chain)
 {
   int i;
-  int length = (chain && *chain) ? strlen ((const char *)chain) + 1 : 0;
+  int length = (chain && *chain) ? strlen ((const char *)chain) : 0;
+  if (dat->version <= R_2000 && length)
+    length++;
   bit_write_BS (dat, length);
   for (i = 0; i < length; i++)
     bit_write_RC (dat, (unsigned char)chain[i]);
@@ -1822,7 +1854,7 @@ bit_write_T (Bit_Chain *restrict dat, BITCODE_T restrict s)
 {
   int i, length;
 
-  // only if from r2007+ DWG, not JSON, DXF, add API
+  // only if from r2007+ DWG. not JSON, DXF, add API.
   if (IS_FROM_TU (dat))
     {
       // downconvert TU to TV
@@ -2445,20 +2477,19 @@ bit_TU_to_utf8_len (const BITCODE_TU restrict wstr, const int len)
 */
 char *
 bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
-                const int destlen, const unsigned cquoted)
+                const int destlen, const int srclen, const unsigned cquoted)
 {
   unsigned char c;
   unsigned char *s = (unsigned char *)src;
   const char* endp = dest + destlen;
+  const unsigned char* ends = src + srclen;
   char *d = dest;
 
   while ((c = *s++))
     {
       if (dest >= endp)
-        {
-          return NULL;
-        }
-      else if (cquoted && c == '\\' && dest+1 < endp &&
+        return NULL;
+      else if (cquoted && c == '\\' && dest+1 < endp && s + 1 <= ends &&
           // skip \" to " and \\ to \.
           (*s == '"' || *s == '\\' || *s == 'r' || *s == 'n'))
         {
@@ -2474,7 +2505,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
               }
         }
       // \uxxxx => \U+XXXX as in bit_embed_TU
-      else if (c == '\\' && dest+7 < endp && *s == 'u')
+      else if (c == '\\' && dest+7 < endp && *s == 'u' && s + 5 <= ends)
         {
           *dest++ = c;
           *dest++ = 'U';
@@ -2489,7 +2520,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
         {
           *dest++ = c;
         }
-      else if ((c & 0xe0) == 0xc0)
+      else if ((c & 0xe0) == 0xc0 && s + 1 <= ends)
         {
           /* ignore invalid utf8 for now */
           if (dest+7 < endp)
@@ -2510,7 +2541,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
       else if ((c & 0xf0) == 0xe0)
         {
           /* warn on invalid utf8 */
-          if (dest+2 < endp &&
+          if (dest+2 < endp && s + 1 <= ends &&
               (*s < 0x80 || *s > 0xBF || *(s+1) < 0x80 || *(s+1) > 0xBF))
             {
               LOG_WARN ("utf-8: BAD_CONTINUATION_BYTE %s", s);
@@ -2519,7 +2550,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
             {
               LOG_WARN ("utf-8: NON_SHORTEST %s", s);
             }
-          if (dest+7 < endp)
+          if (dest+7 < endp && s + 1 <= ends)
             {
               BITCODE_RS wc = ((c & 0x0f) << 12) | ((*s & 0x3f) << 6) | (*(s+1) & 0x3f);
               *dest++ = '\\';
@@ -2532,9 +2563,13 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
             }
           else
             return NULL;
+          if (s + 2 > ends)
+            break;
           s++;
           s++;
         }
+      if (s >= ends)
+        break;
       /* everything above 0xf0 exceeds ucs-2, 4-6 byte seqs */
     }
 
@@ -2548,6 +2583,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
 /** converts UTF-8 to UCS-2. Returns a copy.
     TODO: unquote json_cquote as above.
  */
+ATTRIBUTE_MALLOC
 BITCODE_TU
 bit_utf8_to_TU (char *restrict str, const unsigned cquoted)
 {
@@ -2609,6 +2645,8 @@ bit_utf8_to_TU (char *restrict str, const unsigned cquoted)
 int
 bit_eq_T (Bit_Chain *restrict dat, const BITCODE_T restrict wstr1, const char *restrict str2)
 {
+  if (!wstr1 || !str2)
+    return str2 == wstr1;
   if (IS_FROM_TU (dat))
     return bit_eq_TU (str2, (BITCODE_TU)wstr1);
   else
@@ -2984,29 +3022,34 @@ bit_chain_init_dat (Bit_Chain *restrict dat, const int size,
  * Allocates or adds more memory space for bit_chain
  * adds a 1kB page.
  */
-#define CHAIN_BLOCK 1024
-void
-bit_chain_alloc (Bit_Chain *dat)
+void bit_chain_alloc_size (Bit_Chain *dat, const size_t size)
 {
   if (dat->size == 0)
     {
-      bit_chain_init (dat, CHAIN_BLOCK);
+      bit_chain_init (dat, size);
     }
   else
     {
       dat->chain
-          = (unsigned char *)realloc (dat->chain, dat->size + CHAIN_BLOCK);
+          = (unsigned char *)realloc (dat->chain, dat->size + size);
       if (!dat->chain)
         {
           loglevel = dat->opts & DWG_OPTS_LOGLEVEL;
           LOG_ERROR ("Out of memory")
           abort();
         }
-      memset (&dat->chain[dat->size], 0, CHAIN_BLOCK);
-      dat->size += CHAIN_BLOCK;
+      memset (&dat->chain[dat->size], 0, size);
+      dat->size += size;
     }
 }
 
+#define CHAIN_BLOCK 4096
+void
+bit_chain_alloc (Bit_Chain *dat)
+{
+  bit_chain_alloc_size (dat, CHAIN_BLOCK);
+}
+
 void
 bit_chain_free (Bit_Chain *dat)
 {
@@ -3209,8 +3252,6 @@ bit_calc_CRC (const uint16_t seed, unsigned char *addr, long len)
 uint32_t
 bit_calc_CRC32 (const uint32_t seed, unsigned char *addr, long len)
 {
-  unsigned char al;
-  uint32_t dx = ~seed; /* inverted */
 
   static const uint32_t crctable[256] = {
     0x00000000, 0x77073096, 0xee0e612c, 0x990951ba, 0x076dc419, 0x706af48f,
@@ -3258,13 +3299,12 @@ bit_calc_CRC32 (const uint32_t seed, unsigned char *addr, long len)
     0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
   };
 
-  for (; len > 0; len--)
-    {
-      al = (unsigned char)((*addr) ^ ((unsigned char)(dx & 0xFF)));
-      dx = ((dx >> 8) & 0xFF) ^ crctable[al];
-      addr++;
-    }
-  return ~dx;
+  uint32_t invertedCrc = ~seed;
+  while (len--) {
+    uint8_t byte = *addr++;
+    invertedCrc = (invertedCrc >> 8) ^ crctable[(invertedCrc ^ byte) & 0xff];
+  }
+  return ~invertedCrc;
 }
 
 bool does_cross_unicode_datversion (Bit_Chain *restrict dat)
@@ -3276,7 +3316,8 @@ bool does_cross_unicode_datversion (Bit_Chain *restrict dat)
     return false;
 }
 
-/* Copy the whole content of tmp_data to dat, and reset tmp_dat. */
+/* Copy the whole content of tmp_data to dat, and reset tmp_dat.
+   WARN: This might change dat->chain  */
 void bit_copy_chain (Bit_Chain *restrict dat, Bit_Chain *restrict tmp_dat)
 {
   unsigned long i;
@@ -3287,6 +3328,7 @@ void bit_copy_chain (Bit_Chain *restrict dat, Bit_Chain *restrict tmp_dat)
   // check if dat is byte aligned, tmp_dat always is. we can use memcpy then.
   if (!dat->bit)
     {
+      assert(!tmp_dat->bit);
       memcpy (&dat->chain[dat->byte], &tmp_dat->chain[0], size);
       dat->byte += size;
     }
