@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009-2020 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2009-2021 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -76,6 +76,7 @@ typedef struct _bit_chain
 // only if from r2007+ DWG, not JSON, DXF, add API
 #define IS_FROM_TU(dat) (dat->from_version >= R_2007) && !(dat->opts & DWG_OPTS_IN)
 #define IS_FROM_TU_DWG(dwg) (dwg->header.from_version >= R_2007) && !(dwg->opts & DWG_OPTS_IN)
+#define TU_to_int(b) ((b[1] << 8) + b[0])
 
 /* Functions for raw data manipulations.
  */
@@ -194,6 +195,7 @@ void normalize_BE (BITCODE_3BD ext);
 
 BITCODE_DD bit_read_DD (Bit_Chain *dat, double default_value);
 BITCODE_BB bit_write_DD (Bit_Chain *dat, double value, double default_value);
+int bit_eq_DD (double value, double default_value);
 
 BITCODE_BT bit_read_BT (Bit_Chain *dat);
 void bit_write_BT (Bit_Chain *dat, double value);
@@ -290,7 +292,7 @@ EXPORT char *bit_TU_to_utf8_len (const BITCODE_TU restrict wstr, const int len)
     Returns NULL if not enough room in dest. */
 EXPORT char *
 bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
-                const int len, const unsigned cquoted);
+                const int destlen, const int srclen, const unsigned cquoted);
 
 /** Converts UTF-8 to UCS-2. Returns a copy.
     Needed by dwg importers, writers (e.g. dxf2dwg)
@@ -333,6 +335,7 @@ void bit_chain_init (Bit_Chain *dat, const int size);
 void bit_chain_init_dat (Bit_Chain *restrict dat, const int size,
                          const Bit_Chain *restrict from_dat);
 void bit_chain_alloc (Bit_Chain *dat);
+void bit_chain_alloc_size (Bit_Chain *dat, const size_t size);
 void bit_chain_free (Bit_Chain *dat);
 // after bit_chain_init
 #define bit_chain_set_version(to, from)                                       \
