@@ -513,13 +513,24 @@ decompress_r2007 (BITCODE_RC *restrict dst, const unsigned dst_size,
       if (length == 0)
         length = read_literal_length (&src, opcode);
 
-      if ((dst + length) > dst_end || (src + length) > src_end)
+      if ((dst + length) >= dst_end || (src + length) > src_end)
         {
+          if (DWG_LOGLEVEL >= DWG_LOGLEVEL_HANDLE)
+            {
+              if ((dst + length) >= dst_end)
+                HANDLER(OUTPUT, "copy_compressed_bytes: dst %p + %u >= %p\n",
+                        dst, (unsigned)length, dst_end);
+              else
+                HANDLER(OUTPUT, "copy_compressed_bytes: src %p + %u > %p\n",
+                        src, (unsigned)length, src_end);
+            }
           LOG_ERROR ("Decompression error: length overflow");
           return DWG_ERR_INTERNALERROR;
         }
 
-      LOG_INSANE("copy_compressed_bytes (%p, %p, %u)\n", dst, src, length);
+      LOG_INSANE (
+          "copy_compressed_bytes (%p, %p, %u). remaining src: %ld, dst: %ld\n",
+          dst, src, (unsigned)length, (long)(src_end - src), (long)(dst_end - dst));
       copy_compressed_bytes (dst, src, length);
 
       dst += length;
@@ -538,15 +549,17 @@ decompress_r2007 (BITCODE_RC *restrict dst, const unsigned dst_size,
         {
           if ((dst + length) > dst_end)
             {
+              LOG_HANDLE("copy_bytes: dst %p + %u > %p\n", dst, (unsigned)length, dst_end);
               LOG_ERROR ("Decompression error: length overflow");
               return DWG_ERR_INTERNALERROR;
             }
           if (offset > (uint32_t) (dst - dst_start))
             {
+              LOG_HANDLE("copy_bytes: offset %u > %p - %p\n", (unsigned)offset, dst, dst_start);
               LOG_ERROR ("Decompression error: offset underflow");
               return DWG_ERR_INTERNALERROR;
             }
-          LOG_INSANE ("copy_bytes (%p, %u, %u)\n", dst, length, offset);
+          LOG_INSANE ("copy_bytes (%p, %u, [%u])\n", dst, (unsigned)length, (unsigned)offset);
           copy_bytes (dst, length, offset);
 
           dst += length;
@@ -655,6 +668,8 @@ read_system_page (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
       LOG_ERROR ("Invalid r2007 system page: page_size: %" PRId64, page_size);
       return NULL;
     }
+  LOG_HANDLE ("read_system_page: size_comp: %" PRId64 ", size_uncomp: %" PRId64
+                 ", repeat_count: %" PRId64 "\n", size_comp, size_uncomp, repeat_count);
   assert ((uint64_t)size_comp < dat->size);
   assert ((uint64_t)size_uncomp < dat->size);
   assert ((uint64_t)repeat_count < DBG_MAX_COUNT);
@@ -678,7 +693,7 @@ read_system_page (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
     }
 
   if (size_comp < size_uncomp)
-    error = decompress_r2007 (data, size_uncomp, pedata, MIN (pedata_size, size_comp));
+    error = decompress_r2007 (data, size_uncomp + 1, pedata, MIN (pedata_size, size_comp));
   else
     memcpy (data, pedata, size_uncomp);
 
@@ -725,7 +740,7 @@ read_data_page (Bit_Chain *restrict dat, BITCODE_RC *restrict decomp,
     }
 
   if (size_comp < size_uncomp)
-    error = decompress_r2007 (decomp, size_uncomp, pedata,
+    error = decompress_r2007 (decomp, size_uncomp + 1, pedata,
                               MIN (pedata_size, size_comp));
   else
     memcpy (decomp, pedata, size_uncomp);
@@ -802,6 +817,13 @@ read_data_section (Bit_Chain *sec_dat, Bit_Chain *dat,
                      (long)section_page->offset, (long)max_decomp_size)
           return DWG_ERR_VALUEOUTOFBOUNDS;
         }
+      if (max_decomp_size < section_page->uncomp_size)
+        {
+          free (decomp);
+          LOG_ERROR ("Invalid section size %ld < %ld",
+                     (long)max_decomp_size, (long)section_page->uncomp_size)
+          return DWG_ERR_VALUEOUTOFBOUNDS;
+        }
 
       dat->byte = page->offset;
       // only if compressed. TODO: Isn't there a compressed flag as with 2004+?
@@ -820,6 +842,13 @@ read_data_section (Bit_Chain *sec_dat, Bit_Chain *dat,
         }
       else
         {
+          if (section_page->uncomp_size > max_decomp_size - section_page->offset ||
+              section_page->uncomp_size > dat->size - dat->byte)
+            {
+              free (decomp);
+              LOG_ERROR ("Invalid section size %ld", (long)section_page->uncomp_size);
+              return DWG_ERR_VALUEOUTOFBOUNDS;
+            }
           memcpy (&decomp[section_page->offset], &dat->chain[dat->byte],
                   section_page->uncomp_size);
         }
@@ -923,7 +952,8 @@ read_sections_map (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
 #if 1
       /* compressed */
       if (section->data_size > 10 * dat->size
-          || section->name_length >= (int64_t)dat->size)
+          || section->name_length >= (int64_t)dat->size
+          || section->name_length >= 48)
         {
           LOG_ERROR ("Invalid System Section");
           free (section);
@@ -1214,8 +1244,8 @@ read_file_header (Bit_Chain *restrict dat,
   LOG_TRACE ("len2:          %d\n", (int)len2);      // 0 when compressed
 
   if (compr_len > 0)
-    error = decompress_r2007 ((BITCODE_RC *)file_header, 0x110, &pedata[32],
-                              MIN (compr_len, pedata_size - 32));
+    error = decompress_r2007 ((BITCODE_RC *)file_header, sizeof (r2007_file_header) + 1,
+                              &pedata[32], MIN (compr_len, pedata_size - 32));
   else
     memcpy (file_header, &pedata[32], sizeof (r2007_file_header));
 
